-module(tb_client).
-export([start/3]).
-compile([debug_info]).

%% Opts = [opt()]
%% opt() = {tcp_opts, [tcp_opt()]} | {blksize, Bytes} | {time_limit, int()|infinity}
start(SvrHost, SvrPort, Opts) ->
    spawn(fun() -> start_client(SvrHost, SvrPort, Opts) end).

start_client(Host, Port, Opts) ->
    process_flag(trap_exit, true),
    TcpOpts = make_tcp_opts(Opts),
    TimeLimit = tb_util:get_opt(time_limit, 10000, Opts),
    Size = tb_util:get_opt(blksize, 65535, Opts),
    Block = make_block(Size),
    Self = self(),
    case gen_tcp:connect(Host, Port, TcpOpts) of
        {ok, Sock} ->
            case TimeLimit of
                infinity ->
                    ok;
                MilliSecs when is_integer(MilliSecs) ->
                    erlang:send_after(MilliSecs, Self, {Self, expired, MilliSecs})
            end,
            loop(Sock, Block, 0, now());
        Error ->
            erlang:error(Error)
    end.

loop(Sock, Block, BytesSent, StartTime) ->
    Self = self(),
    receive
        {Self, expired, Ms} ->
            io:format("Finished (~p ms time limit expired).\n", [Ms])
    after
        0 ->
            ok = gen_tcp:send(Sock, Block),
            loop(Sock, Block, BytesSent + byte_size(Block), StartTime)
    end.

make_tcp_opts(Opts) when is_list(Opts) ->
    TcpOpts = tb_util:get_opt(tcp_opts, [], Opts),
    SrtDefOpts = lists:keysort(1, [
        {packet, raw},
        {packet_size, 65536},
        {sndbuf, 1024 * 1024},
        {send_timeout, 3000}
    ]),
    SrtOpts = lists:keysort(1, TcpOpts),
    lists:ukeymerge(1, SrtOpts, SrtDefOpts).

make_block(Size) ->
    <<0:Size/integer-unit:8>>.

