-module(tb_util).
-export([stats/3, format_stats/1, get_opt/3]).

-define(MILLION, 1000000).

stats(Start, End, BytesRecv) ->
    ElapsedSecs = (now_to_micros(End) - now_to_micros(Start)) / ?MILLION,
    MBytesRecv = BytesRecv / (1024.0 * 1024.0),
    XferRate = MBytesRecv / ElapsedSecs, % MB/s
    {stats, {ElapsedSecs, XferRate, MBytesRecv}}.

format_stats({stats, {ElapsedSecs, XferRate, MBytesRecv}}) ->
    lists:flatten(io_lib:format("~.2f MB received in ~.2f secs (~.2f MB/sec)\n", [MBytesRecv, ElapsedSecs, XferRate])).

now_to_micros({MS, S, US}) ->
    MS * ?MILLION * ?MILLION +
    S * ?MILLION +
    US.

get_opt(Opt, DefVal, L) ->
    case lists:keysearch(Opt, 1, L) of
        {value, {_, Val}} ->
            Val;
        false ->
            DefVal
    end.

